# ----------------------------------------------------------------------------
# Csound Tcl-Music-Script                                          SEP-21-2002
# Copyright (c) 2002 by Hartmut Schwarz               matut@matut-spielwerk.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# Grundwissen der Musik fuer den Tcl-Interpreter, Lektion (1) + (2)
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
puts "Lektion x : 123-Notation, 123-notes";
# ----------------------------------------------------------------------------

# **********************************
# I. Beschreibung der 123-Notation *
# **********************************

# 1. Tastennummern - Notenlinien
# ------------------------------

# a -   21       -
# g     19
# f ----17--------
# e     16
# d ----14--------
# c     12 
# h ----11--------
# a      9           (440 Hz bei kybs = 12)
# g -----7--------
# f      5
# e -----4--------
# d      2
# c -    0       -   (c1 = 8.00 bei kybs = 12)

# 2. Noten- und Pausenwerte
# -------------------------

# 12____ =  1/1
# 12___  =  3/4
# 12__   =  1/2
# 12_    =  1/4
# 12.... =  1/4
# 12...  = 3/16
# 12..   =  1/8
# 12.    = 1/16
# 12'''' = 1/16
# 12'''  = 3/64
# 12''   = 1/32
# 12'    = 1/64
# ( 12   = 0 )

# einfacher gesagt:

# '_' = + 1/4
# '.' = + 1/16
# "'" = + 1/64
# Pausen mit P anstelle der Zahl

# *******************************
# II. Die 123-Notenleseprozedur *
# *******************************

proc read_notes { rCTX pass_proc } {
# ----------------------------------------------------------------------------
# Liest Noten in der 123-Notation, wie fuer die abc-Notation beschrieben.
# ----------------------------------------------------------------------------
 # --- Das Spiel vorbereiten, d.h. Initialisierungen ---
 set indx 0;                  # Index in Notenliste
 set crtv  [clock_get_tval $rCTX]; # Aktuelle Zeit als Notenwert
 set nline [TREEget $rCTX N]; # Notenlinie, ...
 set mline [TREEget $rCTX M]; # ... Metalinie und ...
 set cline [TREEget $rCTX C]; # ... Akkordlinie aus Kontext holen

 # --- Anfangkennung senden ---
 set crtv [$pass_proc $rCTX "begin" $crtv 0 "P" "." "."];

 # --- Noten lesen, d.h. Schleife ueber die Notenliste ---
 # foreach {nkey nval} $notes {}
 foreach note $nline {
  if { [parse_nkey $note nkey nval] == 1 } {
   if { $nval <= 0 } {
    puts "*** Fehler: Abbruch wegen Notenwert 0 ***";
    break;
   }
   set meta  [get_meta_string $indx $mline]; 
   set csym [get_meta_string $indx $cline];
   set crtv [$pass_proc $rCTX $indx $crtv $nval $nkey $meta $csym];
   incr indx;
  }
 }
 
 # --- Endekennung senden ---
 set crtv [$pass_proc $rCTX "end" $crtv 0 "P" "." "."];

 clock_set_tval $rCTX $crtv;
 return $crtv;
} 

proc parse_nkey { note nkey nval } {
# Eine einzelne Note wird in Tastennummer und Notenwert umgewandelt.
 upvar $nkey n;
 upvar $nval tv;
 set result 0;
 set tv 0;
 set len [string length $note];
 if { [scan $note "%d" n] != 1 } {
  set n "P";
 }
 for { set i 1 } { $i < $len } { incr i } {
  set s [string index $note $i];
  if { $s == "_" } {
   incr tv 16;
  } elseif { $s == "." } {
   incr tv 4;
  } elseif { $s == "'" } {
   incr tv 1;
  }
 }
 if { $tv != 0 } {
  set tv [expr $tv / 64.0];
  set result 1;
 }
 return $result;
}
