########################################################################
# Copyright (c) 2004 by Hartmut Schwarz, matut@matut-spielwerk.de      #
#                                                                      #
# This program is free software; you can redistribute it and/or        #
# modify it under the terms of the GNU General Public License          #
# as published by the Free Software Foundation; either version 2       #
# of the License, or (at your option) any later version.               #
#                                                                      #
# This program is distributed in the hope that it will be useful,      #
# but WITHOUT ANY WARRANTY; without even the implied warranty of       #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         #
# GNU General Public License for more details.                         #
#                                                                      #
# You should have received a copy of the GNU General Public            #
# License along with this program; if not, write to the Free           #
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,       #
# MA 02111-1307, USA.                                                  #
########################################################################

# simple functions to get info about  WAV files,
# only working for 44100 smp/s, 16 bit stereo

# get number of samples in a WAV file
proc evalnumsamples { fwav } {
 # open file and set binary mode
 set fid [open $fwav];
 fconfigure $fid -translation binary;
 # read RIFF chunk
 set riffsig [read $fid 4];
 set rifflen [read32BitUnsigned $fid];
 set wavesig [read $fid 4];
 if { $riffsig != "RIFF" || $wavesig != "WAVE" } {
  puts "\[ERROR\] \"$fwav\" is not a windows WAV sound file";
  close $fid;
  exit 1;
 }
 # read fmt chunk
 set fmt_sig [read $fid 4];
 set fmt_len [read32BitUnsigned $fid];
 set fmt_format_type [read16BitUnsigned $fid];
 set fmt_number_of_channels [read16BitUnsigned $fid];
 set fmt_sample_rate [read32BitUnsigned $fid];
 set fmt_bytes_per_second [read32BitUnsigned $fid];
 set fmt_bytes_per_sample [read16BitUnsigned $fid];
 set fmt_bits_per_sample [read16BitUnsigned $fid]; 
 if { $fmt_sig != "fmt " || $fmt_len != 16 } {
  puts "\[ERROR\] \"$fwav\" has invalid format chunk";
  puts "\[NOTE\] fmt_sig = $fmt_sig";
  puts "\[NOTE\] fmt_len = $fmt_len";
  close $fid;
  exit 1;
 }
 if { $fmt_format_type != 1 || \
      $fmt_number_of_channels != 2 || \
      $fmt_sample_rate != 44100 || \
      $fmt_bytes_per_second != 176400 || \
      $fmt_bytes_per_sample != 4 || \
      $fmt_bits_per_sample != 16 } {
  puts "\[ERROR\] format of \"$fwav\" not supported";
  close $fid;
  exit 1;
 }
 # find data chunk
 set chunkname "";
 set chunklen 0;
 while { $chunkname != "data" } {
  seek $fid 0 current;
  set chunkname [read $fid 4];
  set chunklen [read32BitUnsigned $fid];
 }
 # close file and return number of samples 
 close $fid;
 return [expr $chunklen / $fmt_bytes_per_sample];
}

# get track length of a WAV file in MM:SS format
proc evaltracktime { fwav } {
 if { [catch { evalnumsamples $fwav } result] == 0 } {
  set tt [samples2tracktime $result];
 } else {
  puts $result
  set tt "??:??";
 }
 return $tt
}

# convert number of samples to track length in MM:SS format
proc samples2tracktime { num } {
 return [seconds2tracktime [samples2seconds $num]];
}

# convert track time in seconds to MM:SS format
proc seconds2tracktime { sec } {
 return [format "%02d:%02d" [expr int($sec)/60] [expr int($sec)%60]];
}

# convert number of samples to track duration in seconds
proc samples2seconds { num } {
 set bl 2352.0; # frame block
 return [expr int(round($bl*ceil($num/$bl)/44100.0))]
}

# read a 32 bit integer number
proc read32BitUnsigned { fid } {
 binary scan [read $fid 4] "i" result;
 if { $result < 0 } {
  set result [expr $result + 4294967296];
 }
 return $result; 
}

# read a 16 bit integer number
proc read16BitUnsigned { fid } {
 binary scan [read $fid 2] "s" result;
 if { $result < 0 } {
  set result [expr $result + 65536];
 }
 return $result; 
}
