# ----------------------------------------------------------------------------
# Csound Tcl Partitur-Skript      "Kalinka"                        JAN-12-2004
# Copyright (c) 2004 by Hartmut Schwarz               matut@matut-spielwerk.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# ----------------------------------------------------------------------------

# ------------------------------------------- change into source directory ---
cd "[file dirname $argv0]/..";

# ------------------------------------------------ learning basic concepts ---
source "basics/ref.tcl";   # all about reference variables
source "basics/tree.tcl";  # all about data trees

# ------------------------------------------------ learning musical basics ---
source "basics/music.tcl";

# ----------------------------------------------------- special procedures ---

proc volume_sinus { rCTX pifo crespos } {
 global $rCTX PI2;
 return [expr [TREEget $rCTX vlbs]+$pifo*[TREEget $rCTX fstp]+\
              [TREEget $rCTX cres]*sin($PI2*([TREEget $rCTX VPHA]\
                                             +$crespos/[TREEget $rCTX VLEN]))];
}

proc key_sinus { rCTX key pos } {
 global $rCTX PI2;
 azimuth_step $rCTX $pos;
 if { [string is double $key] } {
  return [expr $key+[TREEget $rCTX KAMP]*sin($PI2*([TREEget $rCTX KPHA]+$pos/[TREEget $rCTX KLEN]))];
 } else {
  return $key;
 }
}

proc azimuth_step { rCTX pos } {
 TREEset $rCTX iazm [expr int([TREEget $rCTX APHA]+$pos*[TREEget $rCTX AVEL])%360]
}

proc prepoly_up {} {
# prelude procedure calculates the acceleration timepoly coefficients
 upvar 1 rCTX rCTX;
 set TMTV0 [TREEget $rCTX TMTV0];
 set TM1TV [expr $TMTV0*[TREEget $rCTX tmtv]];
 set TM2TV [expr (1.0/$TMTV0-1.0)*$TM1TV/(2.0*[TREEget $rCTX chtv])];
 TREEset $rCTX TM1TV $TM1TV;
 TREEset $rCTX TM2TV $TM2TV;
 return;
}

proc prepoly_down {} {
# postlude procedure calculates the retardation timepoly coefficients
 upvar 1 rCTX rCTX;
 set TMTV0 [TREEget $rCTX TMTV0];
 set TM1TV [TREEget $rCTX tmtv];
 set TM2TV [expr ($TMTV0-1.0)*$TM1TV/(2.0*[TREEget $rCTX chtv])];
 TREEset $rCTX TM1TV $TM1TV;
 TREEset $rCTX TM2TV $TM2TV;
 return;
}

proc timepoly { rCTX nval rtval rtime rdura } {
# Zeitfunktion schreibt Zeit fort mittels eines Polynoms zweiten
# Grades, dessen Koeffizienten in einer der Vorspielprozeduren
# prepoly_up oder prepoly_down berechnet wurden. Die Notendauer
# bleibt gegenueber der Standardprozedur time_function unveraendert.
 global $rCTX;
 upvar $rtval tval $rtime time $rdura dura;
 set tval [expr [clock_get_tval $rCTX] + $nval]; # wie gehabt
 set tv [expr $tval - [clock_get_stamp_tv $rCTX]];
 set time [expr [clock_get_stamp_tm $rCTX]\
              + [TREEget $rCTX TM1TV]*$tv\
              + [TREEget $rCTX TM2TV]*pow($tv,2.0)];
 set tmtv [TREEget $rCTX tmtv];
 set dura [expr $nval*$tmtv]; # wie gehabt
 return ;
}

# ------------------------------------------------------ das Spiel beginnt ---
set root [plant]; # einen neuen Klangbaum pflanzen
# Kalinka
set stem [TREEnew $root {crtv 0 kybs 12 chtv 0.5 vlbs 68 vstp 1.026 fstp 4 chbt ./pp./pp.\
          AVEL -360 APHA 0 TMTV0 1.5 KAMP 1 KPHA 0.25 KLEN 2.0 VPHA 0.25 VLEN 2.0 CMOD f::,.;\
          Z2 <.*>63. Z1 <.*>31! N2 <.*>15! N4 <.*>7! N8 <.*>3! N80 f.p;;; N16 <.*>1!\
          Z3N4 <.*>23! Z3N8 <.*>11! Z3N16 <.*>5! P4 :;;;;;;; P8 :;;;}];
sprout $stem\
 {chbt . csym (11) more {\
  {cmod <Z2> chbt . text "Kaaaa...-" chtv 2.0 tmfn timepoly prld prepoly_up\
   isze 0 iazm 0 TMTV0 16 KAMP 13.5 kybs 25.5 vlbs 70 cres 2.0\
   vlfn volume_sinus kyfn key_sinus}\
  {rept 2 cmod <N8> chtv 0.125 text "Kaaaa...-"}\
 }}\
 {N80 <N8> HMOD1 '''.;p,,.: HMOD2 pu..''.. EMOD1 ''.;p,.: EMOD2 pu..'.. rept 27 more {\
  {song {} vars {\
   {tmtv 2.00}\
   {tmtv 1.59}\
   {tmtv 1.26}\
  }}\
  {velo 1 csym (9,6,7)/(16,13,10,6)/(16,13,10,6)   cmod <N4><N80><N8>/<HMOD1>/<HMOD2>        text "linka, Ka-"}\
  {velo 2 csym (9,6,7)/(16,13,10,6)/(16,13,10,6)   cmod <N4><N80><N8>/<HMOD1>/<HMOD2>        text "linka, Ka-"}\
  {velo 3 csym (9,7,6)/(16,13,10,6)/(16,13,10,6)   cmod <N4><N80><N8>/<HMOD1>/<HMOD2>        text "linka, ma-"}\
  {velo 4 csym (4,11,11)/(11,7,4)/(11,7,4)         cmod <N4><N8><N8>/<EMOD1>/<EMOD2>         text "ja, fsadu"}\
  {velo 5 csym (9,7,6,7)/(16,13,10,6)/(16,13,10,6) cmod <Z3N16><N16><N8><N8>/<HMOD1>/<HMOD2> text "jagoda ma-"}\
  {velo 6 csym (9,6,7)/(16,13,10,6)/(16,13,10,6)   cmod <N4><N80><N8>/<HMOD1>/<HMOD2>        text "linka, ma-"}\
  {velo 7 csym (9,7,6)/(16,13,10,6)/(16,13,10,6)   cmod <N4><N80><N8>/<HMOD1>/<HMOD2>        text "linka, ma-"}\
  {more {\
   {rept 2 velo 8 csym (4,11,11)/(11,7,4)/(11,7,4) cmod <N8><N4><N8>/<EMOD1>/<EMOD2> text "ja, ach, Ka-"}\
   {csym (4,16)/(7,11)/(7,11) cmod <N8><Z2>/<.>34/<.>34\
    text "jaaaaaaa..." chtv 2.125 tmtv 2.0 tmfn timepoly\
    iazm 0 prld prepoly_down KAMP 0.5\
    kybs 11.5 kyfn key_sinus}\
  }}\
 }}\
 {csym (14)/(6,9)/(6,9) cmod <Z2>/<.>32/<.>32\
  text "....aaaaa...." chtv 2.0 tmfn timepoly\
  iazm 0 prld prepoly_up\
  KAMP 0.5 kybs 12.5 kyfn key_sinus}\
 {csym (11,14,12,11,9)/(7,11)/(7,11) cmod <N8><N8><N8><N16><N16>/u..../,, text "Pat ssa-"}\
 {csym (7,2)/(11,7)/(11,7)           cmod <N4><N4>/u....,,                text "snoju,"}\
 {csym (11,14,12,11,9)/(7,11)/(7,11) cmod <N8><N8><N8><N16><N16>/u..../,, text "pat sielio-"}\
 {csym (7,2)/(11,7)/(11,7)           cmod <N4><N4>/u..../,,               text "noju,"}\
 {csym (4,4,6)/(0,4)/(0,4)           cmod <N4><N8><N8>/u..../,,           text "spatch palla-"}\
 {csym (9,7,6,4)/(7,4)/(7,4)         cmod <N8><N8><N8><N8>/u..../,,       text "getie"}\
 {csym (2)/(5,2)/(5,2)               cmod <N4><N4>/u..../,,               text "we mi-"}\
 {csym (2,14)/(7,11)/(7,11)          cmod <N4><N4>/u..../,,               text "nia."}\
 {csym (11,14,9,11)/(7,11)/(7,11)    cmod <N8><N8><N8><N8>/u..../,,       text "Ai ljuli,"}\
 {csym (7,2)/(11,7)/(11,7)           cmod <N4><N4>/u....,,                text "ljuli,"}\
 {csym (11,14,9,11)/(7,11)/(7,11)    cmod <N8><N8><N8><N8>/u..../,,       text "ai ljuli,"}\
 {csym (7,2)/(11,7)/(11,7)           cmod <N4><N4>/u..../,,               text "ljuli,"}\
 {csym (4,4,6)/(0,4)/(0,4)           cmod <N4><N8><N8>/u..../,,           text "spatch palu-"}\
 {csym (9,7,6,4)/(7,4)/(7,4)         cmod <N8><N8><N8><N8>/u..../,,       text "getie"}\
 {csym (14,12)/(11,7)/(11,7)         cmod <N4><N4>/u..../,,               text "we mi-"}\
 {csym (11)/(9,6)/(9,6) more {\
  {rept 2 cmod <Z3N4>/u..;;;;/,, chtv 0.75 vlbs 68 cres -4 text "niaaaaaaaaaa."}\
  {cmod <Z2>/u..;;;;;;;;;;;;;;/,, chtv 2.0 tmfn timepoly prld prepoly_down TMTV0 16\
   isze 0 iazm 0 KPHA -0.25 KAMP 15.0 VPHA -0.25 kybs 27 vlbs 72.5 cres 4.5\
   vlfn volume_sinus kyfn key_sinus text "niaaaaaaaaaa."}\
 }}\
 ;
begin $root "[file rootname [file tail $argv0]]" 0.0; # Anschluss an kadenz
touch $stem 135;
end $root 0.0; # Anschluss an mond 
