# ----------------------------------------------------------------------------
# Csound Tcl Partitur-Skript      "Fingeruebungen"                 NOV-08-2002
# Copyright (c) 2002 by Hartmut Schwarz               matut@matut-spielwerk.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# ----------------------------------------------------------------------------

# ------------------------------------ ins Tcl-Arbeitsverzeichnis wechseln ---
cd "[file dirname $argv0]/..";

# ------------------------------------------- allgemeine Grundlagen lernen ---
source "basics/ref.tcl";   # alles ueber Referenzvariablen
source "basics/tree.tcl";  # alles ueber Datenbaeume

# --------------------------------- die Grundlagen der Csound-Musik lernen ---
source "basics/music.tcl";

# ------------------------------------------------------- Sonderfunktionen ---
proc change {} {
# einige Bestimmungsgroessen neu berechnen
 upvar 1 rCTX rCTX;
 # Tempo neu berechnen
 set tmtv [TREEget $rCTX tmtv];
 set tmtv [expr $tmtv*0.912082941];
 if { $tmtv < 0.007 } {
  puts "*** tmtv limit reached : $tmtv < 0.007 ***";
  exit 1;
 } else {
  TREEset $rCTX tmtv $tmtv;
 }
 # Grundton neu berechnen
 set kybs [TREEget $rCTX kybs];
 set kybs [expr $kybs*1.026764960];
 if { $kybs > 73 } {
  puts "*** kybs limit reached : $kybs > 73 ***";
  exit 1;
 } else {
  TREEset $rCTX kybs $kybs;
 }
 # Anschlagstaerke neu berechnen
 set vlbs [TREEget $rCTX vlbs];
 set vlbs [expr $vlbs*0.998743347];
 if { $vlbs < 73 } {
  puts "*** vlbs limit reached : $vlbs < 73 ***";
  exit 1;
 } else {
  TREEset $rCTX vlbs $vlbs;
 }
 # Stakkato neu berechnen
 set tvtv [TREEget $rCTX tvtv];
 set tvtv [expr $tvtv*1.01124252];
 if { $tvtv > 1.1 } {
  puts "*** tvtv limit reached : $tvtv > 73 ***";
 } else {
  TREEset $rCTX tvtv $tvtv;
 }
 # recalculate rotation parameters
 set DAZ [TREEget $rCTX DAZ];
 set DAN [TREEget $rCTX DAN];
 set DAZ [expr $DAZ*1.119902793];
 set DAN [expr $DAN*1.011427873];
 if { $DAZ > 3 } {
  puts "*** DAZ limit reached : $DAZ > 3 ***";
 } else {
  TREEset $rCTX DAZ $DAZ;
 }
 if { $DAN > 90 } {
  puts "*** DAN limit reached : $DAN > 90 ***";
 } else {
  TREEset $rCTX DAN $DAN;
 }
 return ;
}

# override the play note procedure so that we can change things whenever a single note is played
# (this must be done whenever we've learned a new instrument and forgot the old one in turn)
proc old_learn_instrument [info args learn_instrument] [info body learn_instrument];
proc learn_instrument { rCTX inam } {
 set crin [old_learn_instrument $rCTX $inam];
 if { $crin != $inam } {
  proc old_instr_play_note [info args instr_play_note] [info body instr_play_note];
  proc instr_play_note { rCTX iocn inum crtv nval when dura volm loud nkey meta csym } {
   set DAZ [TREEget $rCTX DAZ];
   set DAN [TREEget $rCTX DAN];
   if { $iocn == 0 } {
    set AZM [TREEget $rCTX AZM0];
    TREEset $rCTX iazm [expr int($AZM+$DAN)%360];
    TREEset $rCTX AZM0 [expr $AZM + $DAZ];
   } else {
    set AZM [TREEget $rCTX AZM1];
    TREEset $rCTX iazm [expr int($AZM-$DAN)%360];
    TREEset $rCTX AZM1 [expr $AZM + $DAZ];
   }
   old_instr_play_note $rCTX $iocn $inum $crtv $nval $when $dura $volm $loud $nkey $meta $csym;
  }
 }
 return $crin;
}

# ------------------------------------------------------ das Spiel beginnt ---
set root [plant]; # einen neuen Klangbaum pflanzen
# Fingeruebungen
set stem [TREEnew $root {kybs 14 vlbs 80 tmtv 2.0 tvtv 0.5 chbt ./f. \
          idst 200 AZM0 0 AZM1 0 DAZ 0.01 DAN 45}]
sprout $stem\
{rept 25 more {\
 {song {} vars {\
  {cmod .:..:..:..:./''''.:..:..:..:.}\
  {cmod ''''.;.;...;.;../.;.;...;.;..}\
  {cmod .;...;.;...;/''''.;...;.;...;}\
  {cmod ......../''''........}\
  {cmod ''''......../........}\
 }}\
 {csym (C,D,E,F,G,F,E,D)/(C,D,E,F,G,F,E,D)}\
 {csym (C,E,D,F,G,E,F,D)/(C,E,D,F,G,E,F,D)}\
 {csym (G,F,E,F,E,D,C,E)/(G,F,E,F,E,D,C,E)}\
 {csym (E,F,E,F,D,C,G,F)/(E,F,E,F,D,C,G,F)}\
}} {song change};
begin $root "[file rootname [file tail $argv0]]" 0.5;
touch $stem 1586;
end $root 0.2;
