# ----------------------------------------------------------------------------
# Csound Tcl Partitur-Skript "Bunt sind schon die Waelder"         NOV-12-2003
# Copyright (c) 2002 by Hartmut Schwarz               matut@matut-spielwerk.de
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307, USA.
#
# ----------------------------------------------------------------------------

# ------------------------------------ ins Tcl-Arbeitsverzeichnis wechseln ---
cd "[file dirname $argv0]/..";

# ------------------------------------------- allgemeine Grundlagen lernen ---
source "basics/ref.tcl";   # alles ueber Referenzvariablen
source "basics/tree.tcl";  # alles ueber Datenbaeume

# --------------------------------- die Grundlagen der Csound-Musik lernen ---
source "basics/music.tcl";

# ------------------------------------------------------- Sonderfunktionen ---
proc volumewave { rCTX pifo crespos } {
# Wellenfoermige Anschlagfunktion folgenden Bestimmungsgroessen:
# VAMP = Anschlagwellen-Amplitude in Vielfachen von tmtv
# VPHA = Anschlagwellen-Phase in Vielfachen von chtv
# VLEN = Anschlagwellen-Laenge in Vielfachen von chtv
 global $rCTX, PI2;
 return [expr [volume_function $rCTX $pifo $crespos]\
  + [TREEget $rCTX VAMP]*sin(  $PI2*(  [TREEget $rCTX VPHA]\
                                     + [expr $crespos+[clock_get_tval $rCTX]\
                                     / [TREEget $rCTX chtv]])\
                             / [TREEget $rCTX VLEN])];
}

proc keywave { rCTX key pos } {
# Wellenfoermige Grundtonfunktion folgenden Bestimmungsgroessen:
# KAMP = Grundtonwellen-Amplitude in Vielfachen von tmtv
# KPHA = Grundtonwellen-Phase in Vielfachen von chtv
# KLEN = Grundtonwellen-Laenge in Vielfachen von chtv
 global $rCTX, PI2;
 if { [string is double $key] } {
  set key [expr [key_function $rCTX $key $pos]\
   + [TREEget $rCTX KAMP]*sin(  $PI2*(  [TREEget $rCTX KPHA]\
                                      + [expr $pos+[clock_get_tval $rCTX]\
                                      / [TREEget $rCTX chtv]])\
                              / [TREEget $rCTX KLEN])];
 }
 return $key;
}

proc timewave { rCTX nval rtval rtime rdura } {
# Wellenfoermige Zeitfortschreibung mit folgenden Bestimmungsgroessen:
# TIM0 = Anfangszeit fuer Notenwert 0
# TAMP = Zeitwellen-Amplitude in Vielfachen von tmtv
# TPHA = Zeitwellen-Phase in Vielfachen von chtv
# TLEN = Zeitwellen-Laenge in Vielfachen von chtv
 global $rCTX, PI2;
 upvar $rtval tval $rtime time $rdura dura;
 # aktuellen Notenwert holen und neuen Wert berechnen
 set crtv [clock_get_tval $rCTX];
 set tval [expr $crtv + $nval];
 # Bestimmungsgroessen holen
 set tmtv [TREEget $rCTX tmtv];
 set chtv [TREEget $rCTX chtv];
 set TIM0 [TREEget $rCTX TIM0];
 set TAMP [TREEget $rCTX TAMP];
 set tpha [expr [TREEget $rCTX TPHA]*$chtv];
 set tlen [expr [TREEget $rCTX TLEN]*$chtv];
 # neue Zeit unabhaengig von aktueller Zeit berechnen
 set time [expr $TIM0 + $tmtv*($tval + $TAMP*sin($PI2*($tpha+$tval/$tlen)))];
 # Notendauer unabhaengig von aktueller Zeit berechnen
 # 1-ter Wert aus Ableitung 
 set dura1 [expr $tmtv*(1 + $TAMP*cos($PI2*($tpha+$tval/$tlen)))*$nval];
 # 2-ter Wert aus Zeitdifferenz
 set crtm [expr $TIM0 + $tmtv*($crtv + $TAMP*sin($PI2*($tpha+$crtv/$tlen)))];
 set dura2 [expr $time - $crtm];
 # von beiden Werten den laengeren nehmen damit das Spiel nicht abgehackt wirkt
 if { $dura1 > $dura2 } {
  set dura $dura1;
 } else {
  set dura $dura2;
 }
 return ;
}

# ------------------------------------------------------ das Spiel beginnt ---
set root [plant]; # einen neuen Klangbaum pflanzen
# Bunt sind schon die Waelder
set stem [TREEnew $root {chtv 0.75\
          TIM0 1 TAMP 0.1 TPHA 0.0 TLEN 24 tmfn timewave\
          VAMP 0.0 VPHA 0.0 VLEN 24 vlfn volumewave\
          KAMP 0.0 KPHA 0.0 KLEN 24 kyfn keywave}];
sprout $stem\
 {more {\
  {song {} vars {\
   {KAMP  0.40 TAMP 0.4 VAMP  0.0 CRTV  0 vlbs 66 fstp 2.0 Z  3}\
   {KAMP  0.00 TAMP 0.2 VAMP  0.0 CRTV  9 vlbs 68 fstp 3.0 Z  4}\
   {KAMP  0.00 TAMP 0.0 VAMP  0.0 CRTV 18 vlbs 68 fstp 0.0 Z  A}\
  }}\
  {rept 52 more {\
   {song {} vars {\
    {KLEN 24.0 TLEN 24.0 VLEN 24.0 kybs -12 crtv {[STEM CRTV]} chbt pp./pp.}\
    {KLEN 12.0 TLEN 12.0 VLEN 12.0 kybs   0 crtv {[STEM CRTV]} chbt p./p.}\
    {KLEN  6.0 TLEN  6.0 VLEN  6.0 kybs  12 crtv {[STEM CRTV]} chbt ./.}\
    {KLEN  4.0 TLEN  4.0 VLEN  4.0 kybs  24 crtv {[STEM CRTV]} chbt f./f.}\
   }}\
   {rept 12 more {\
    {csym (7,7,7,6,7)/(14,11)    cmod .;....|.. text "Bunt sind schon die"}\
    {csym (9,7)/(14,2)           cmod ../..     text "Waelder,"}\
    {csym (11,11,11,9,11)/(14,7) cmod .;....|.. text "gelb die Stoppel-"}\
    {csym (12,11)/(14,11)        cmod ../..     text "felder,"}\
    {csym (9,9,7,11,1)/(14,7)    cmod .;..../.. text "und der Herbst be-"}\
    {csym (2)/(9,6)              cmod .;;;;:/,. text "ginnt."}\
    {csym (11,9,7,6)/(7,1)       cmod .;..;./.. text "Rote Blaetter"}\
    {csym (4)/(7,0)              cmod ../..     text "fallen,"}\
    {csym (12,11,9,11,7)/(4,1)   cmod .;..../.. text "graue Nebel"}\
    {csym (6)/(9,2)              cmod ../..     text "wallen,"}\
    {csym (7,11,14,2,4,6)/(11,6) cmod ....../.. text "kuehler weht der"}\
    {csym (7)/(14,11)            cmod ./,.      text "Wind."}\
   }}\
  }}\
 }}\
 ;
begin $root "[file rootname [file tail $argv0]]" 0.0;
touch $stem 159;
end $root 0.2;
return;
